#region Using declarations
using System;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.ComponentModel;
using System.Xml.Serialization;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

public enum T3CalcMode
{
    Tillson,
    FulksMatulich
}
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
	/// <summary>
	/// T3
	/// </summary>
	[Description("T3 Indicator.")]
	[Gui.Design.DisplayName("T3FulksMatulich")]
	public class T3FulksMatulich : Indicator
	{
		#region Variables
			private double 			amp 		= 0.7;
 			private int 			period		= 14;
			private int 			smooth 		= 0;
			private Color			upColor		= Color.DeepSkyBlue;
			private Color			downColor	= Color.OrangeRed;
			private int				plot0Width	= 2;
			private PlotStyle		plot0Style	= PlotStyle.Line;
			private DashStyle		dash0Style	= DashStyle.Solid;
	   		private T3CalcMode 		calcMode 	= T3CalcMode.FulksMatulich;
			private	DataSeries		e1;
			private	DataSeries		e2;
			private	DataSeries		e3;
			private	DataSeries		e4;
			private	DataSeries		e5;
			private	DataSeries		e6;

	    #endregion

		/// <summary>
		/// This method is used to configure the indicator and is called once before any bar data is loaded.
		/// </summary>
		protected override void Initialize()
		{
            Add(new Plot(Color.FromKnownColor(KnownColor.Green), PlotStyle.Line, "T3Plot"));
            Overlay				= true;
 			PlotsConfigurable	= false;

			e1	= new DataSeries(this);
			e2	= new DataSeries(this);
			e3	= new DataSeries(this);
			e4	= new DataSeries(this);
			e5	= new DataSeries(this);
			e6	= new DataSeries(this);
		}

		/// <summary>
		/// </summary>
		protected override void OnStartUp()
		{
			Plots[0].Pen.Width 		= plot0Width;
			Plots[0].PlotStyle		= plot0Style;
			Plots[0].Pen.DashStyle 	= dash0Style;
		}
		
		/// <summary>
		/// Calculates the indicator value(s) at the current index.
		/// </summary>
		protected override void OnBarUpdate()
		{		
			int smooth=period;

            if (T3CalcMode == T3CalcMode.FulksMatulich)
                smooth = (int) (1+.5*(smooth-1));

			e1.Set(EMA(Close,smooth)[0]);
			e2.Set(EMA(e1,smooth)[0]);
			e3.Set(EMA(e2,smooth)[0]);
			e4.Set(EMA(e3,smooth)[0]);
			e5.Set(EMA(e4,smooth)[0]);
			e6.Set(EMA(e5,smooth)[0]);
			
			double c1=-Math.Pow(amp, 3);
			double c2=(3*Math.Pow(amp, 2))+(3*Math.Pow(amp, 3));
			double c3=(-6*Math.Pow(amp, 2))-(3*amp)-(3*Math.Pow(amp, 3));
            double c4 = 1 + (3 * amp) + (Math.Pow(amp, 3)) + (3 * Math.Pow(amp, 2));
		

			T3Plot.Set((c1*e6[0])+(c2*e5[0])+(c3*e4[0])+(c4*e3[0]));
			if (Rising(T3Plot))
				PlotColors[0][0] = upColor;
			else 
				PlotColors[0][0] = downColor;
			
		}

		#region Properties
		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore()]
		public DataSeries T3Plot
		{
			get { return Values[0]; }
		}

		/// <summary>
		/// </summary>
		[Description("")]
		[Category("Parameters")]
		public int Period
		{
			get { return period; }
			set { period = Math.Max(1, value); }
		}

        [Description("")]
        [Category("Parameters")]
	    public T3CalcMode T3CalcMode
	    {
	        get { return calcMode; }
	        set { calcMode = value; }
	    }

		/// <summary>
		/// </summary>
		[Description("")]
		[Category("Parameters")]
		public double Amplification
		{
			get { return amp; }
			set { amp = Math.Max(0, value); }
		}

		/// <summary>
		/// </summary>
		[XmlIgnore()]
		[Description("Select color for rising T3")]
		[Category("Plots")]
		[Gui.Design.DisplayName("T3 Rising")]
		public Color UpColor
		{
			get { return upColor; }
			set { upColor = value; }
		}
		
		// Serialize Color object
		[Browsable(false)]
		public string UpColorSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(upColor); }
			set { upColor = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
		
		/// <summary>
		/// </summary>
		[XmlIgnore()]
		[Description("Select color for falling T3")]
		[Category("Plots")]
		[Gui.Design.DisplayName("T3 Falling")]
		public Color DownColor
		{
			get { return downColor; }
			set { downColor = value; }
		}
		
		// Serialize Color object
		[Browsable(false)]
		public string DownColorSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(downColor); }
			set { downColor = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
		
 		/// <summary>
		/// </summary>
		[Description("Width for T3P Line.")]
		[Category("Plots")]
		[Gui.Design.DisplayNameAttribute("Line Width")]
		public int Plot0Width
		{
			get { return plot0Width; }
			set { plot0Width = Math.Max(1, value); }
		}

		/// <summary>
		/// </summary>
		[Description("DashStyle for T3 Line")]
		[Category("Plots")]
		[Gui.Design.DisplayNameAttribute("Plot Style")]
		public PlotStyle Plot0Style
		{
			get { return plot0Style; }
			set { plot0Style = value; }
		}
		
		/// <summary>
		/// </summary>
		[Description("DashStyle for T3 Line")]
		[Category("Plots")]
		[Gui.Design.DisplayNameAttribute("Dash Style")]
		public DashStyle Dash0Style
		{
			get { return dash0Style; }
			set { dash0Style = value; }
		}
			    #endregion
	}
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private T3FulksMatulich[] cacheT3FulksMatulich = null;

        private static T3FulksMatulich checkT3FulksMatulich = new T3FulksMatulich();

        /// <summary>
        /// T3 Indicator.
        /// </summary>
        /// <returns></returns>
        public T3FulksMatulich T3FulksMatulich(double amplification, int period, T3CalcMode t3CalcMode)
        {
            return T3FulksMatulich(Input, amplification, period, t3CalcMode);
        }

        /// <summary>
        /// T3 Indicator.
        /// </summary>
        /// <returns></returns>
        public T3FulksMatulich T3FulksMatulich(Data.IDataSeries input, double amplification, int period, T3CalcMode t3CalcMode)
        {
            if (cacheT3FulksMatulich != null)
                for (int idx = 0; idx < cacheT3FulksMatulich.Length; idx++)
                    if (Math.Abs(cacheT3FulksMatulich[idx].Amplification - amplification) <= double.Epsilon && cacheT3FulksMatulich[idx].Period == period && cacheT3FulksMatulich[idx].T3CalcMode == t3CalcMode && cacheT3FulksMatulich[idx].EqualsInput(input))
                        return cacheT3FulksMatulich[idx];

            lock (checkT3FulksMatulich)
            {
                checkT3FulksMatulich.Amplification = amplification;
                amplification = checkT3FulksMatulich.Amplification;
                checkT3FulksMatulich.Period = period;
                period = checkT3FulksMatulich.Period;
                checkT3FulksMatulich.T3CalcMode = t3CalcMode;
                t3CalcMode = checkT3FulksMatulich.T3CalcMode;

                if (cacheT3FulksMatulich != null)
                    for (int idx = 0; idx < cacheT3FulksMatulich.Length; idx++)
                        if (Math.Abs(cacheT3FulksMatulich[idx].Amplification - amplification) <= double.Epsilon && cacheT3FulksMatulich[idx].Period == period && cacheT3FulksMatulich[idx].T3CalcMode == t3CalcMode && cacheT3FulksMatulich[idx].EqualsInput(input))
                            return cacheT3FulksMatulich[idx];

                T3FulksMatulich indicator = new T3FulksMatulich();
                indicator.BarsRequired = BarsRequired;
                indicator.CalculateOnBarClose = CalculateOnBarClose;
#if NT7
                indicator.ForceMaximumBarsLookBack256 = ForceMaximumBarsLookBack256;
                indicator.MaximumBarsLookBack = MaximumBarsLookBack;
#endif
                indicator.Input = input;
                indicator.Amplification = amplification;
                indicator.Period = period;
                indicator.T3CalcMode = t3CalcMode;
                Indicators.Add(indicator);
                indicator.SetUp();

                T3FulksMatulich[] tmp = new T3FulksMatulich[cacheT3FulksMatulich == null ? 1 : cacheT3FulksMatulich.Length + 1];
                if (cacheT3FulksMatulich != null)
                    cacheT3FulksMatulich.CopyTo(tmp, 0);
                tmp[tmp.Length - 1] = indicator;
                cacheT3FulksMatulich = tmp;
                return indicator;
            }
        }
    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// T3 Indicator.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.T3FulksMatulich T3FulksMatulich(double amplification, int period, T3CalcMode t3CalcMode)
        {
            return _indicator.T3FulksMatulich(Input, amplification, period, t3CalcMode);
        }

        /// <summary>
        /// T3 Indicator.
        /// </summary>
        /// <returns></returns>
        public Indicator.T3FulksMatulich T3FulksMatulich(Data.IDataSeries input, double amplification, int period, T3CalcMode t3CalcMode)
        {
            return _indicator.T3FulksMatulich(input, amplification, period, t3CalcMode);
        }
    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// T3 Indicator.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.T3FulksMatulich T3FulksMatulich(double amplification, int period, T3CalcMode t3CalcMode)
        {
            return _indicator.T3FulksMatulich(Input, amplification, period, t3CalcMode);
        }

        /// <summary>
        /// T3 Indicator.
        /// </summary>
        /// <returns></returns>
        public Indicator.T3FulksMatulich T3FulksMatulich(Data.IDataSeries input, double amplification, int period, T3CalcMode t3CalcMode)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.T3FulksMatulich(input, amplification, period, t3CalcMode);
        }
    }
}
#endregion
